require 'spec_helper'

module ClientPages
  describe 'MarkdownContent' do
    subject { ClientPages }

    describe 'invalid path' do
      before { ClientPages.configure { |c| c.content_path = 'not_there' } }
      it { expect(subject.render_content(:bar)).to eql('No such file or directory - bar') }
    end

    describe 'valid path' do
      before { ClientPages.configure { |c| c.content_path = 'spec/fixtures/foo' } }
      it { expect(subject.render_content(:bar)).to eql("<h1>foo</h1>\n\n<p>bar</p>") }
    end

    it 'blends' do
      expect(
        subject.render_content(:test_content) do
          modify :h1, with: { class: 'mycss_class', 'data-foo' => 'bar' }, if: -> { includes?('Content') }
          modify :a, with: { class: 'fancy', title: 'woah' }, if: { href: 'http://somewhere.com' }
          wrap :h1, :div, with: { id: 'my-id', class: 'foo' }
          replace :p, :span, with: { class: 'bar', 'data-p' => 'no-more' }
        end
      ).to eql(
        "<div id=\"my-id\" class=\"foo\"><h1 class=\"mycss_class\" data-foo=\"bar\">Content from "\
        "<a href=\"http://somewhere.com\" class=\"fancy\" title=\"woah\">somewhere</a></h1></div>\n\n"\
        "<span class=\"bar\" data-p=\"no-more\">Lorem do <em>incididunt</em> ut labore</span>\n\n"\
        "<span class=\"bar\" data-p=\"no-more\">Pariatur: "\
        "<a href=\"/bar.com\" title=\"baz\">Foo</a> culpa</span>\n"
      )
    end

    describe 'Renderer' do
      let(:markdown) { fixture('test_content') }

      before do
        ClientPages.configure do |c|
          c.remote = true
          c.caching = false
          c.remote_content_path = 'http://somewhere.com'
        end
        stub_request(:get, 'http://somewhere.com/test_content.md')
          .to_return(body: markdown, status: 200)
      end

      context 'without a block' do
        let(:rendered_content) { subject.render_content(:test_content) }

        it 'renders HTML content for given markdown file from somewhere' do
          expect(rendered_content).to include('<h1>Content from <a href="http://somewhere.com">somewhere</a></h1>')
        end

        describe 'raw markdown' do
          before do
            ClientPages.configure { |c| c.raw_markdown = true }
          end

          it 'renders HTML content for given markdown' do
            expect(subject.render_content('# *raw* markdown')).to eql('<h1><em>raw</em> markdown</h1>')
          end
        end
      end

      context 'with a block' do
        let(:link_list) { fixture('link_list') }
        let(:text_list) { fixture('text_list') }
        before do
          stub_request(:get, 'http://somewhere.com/link_list.md').to_return(body: link_list, status: 200)
          stub_request(:get, 'http://somewhere.com/text_list.md').to_return(body: text_list, status: 200)
        end

        it 'returns a content builder' do
          expect(
            subject.render_content(:test_content) { |c| c }
          ).to be_kind_of(MarkdownContent::ContentBuilder)
        end

        it 'modifies tags and html attributes' do
          expect(
            subject.render_content(:test_content) do
              modify :h1, with: { class: 'mycss_class', 'data-foo' => 'bar' }
              modify :a, with: { class: 'fancy', title: 'woah' }
            end
          ).to include(
            '<h1 class="mycss_class" data-foo="bar">'\
              'Content from <a href="http://somewhere.com" class="fancy" title="woah">somewhere</a>'\
            '</h1>'
          )
        end

        it 'adds attributes conditionally' do
          expect(
            subject.render_content(:test_content) do
              modify :h1, with: { class: 'add_me cool' }, if: true
              modify :a, with: { class: 'add_me_not' }, if: false
            end
          ).to include(
            '<h1 class="add_me cool">Content from <a href="http://somewhere.com">somewhere</a></h1>'
          )
        end

        it 'adds any attribute conditionally and selective' do
          expect(
            subject.render_content(:link_list) do
              modify :li, with: { class: 'yay' }, if: -> { match_any?(:href, '/link1') }
              modify :a, with: { class: 'active' }, if: { href: '/link2' }
              modify :a, with: { class: 'important', 'data-foo' => 'bar' }, if: { title: 'i haz title' }
            end
          ).to include(
            "<li class=\"yay\"><a href=\"/link1\">link1</a></li>\n"\
            "<li><a href=\"/link2\" class=\"active\">link2</a></li>\n"\
            "<li><a href=\"/link3\" title=\"i haz title\" class=\"important\" data-foo=\"bar\">link3</a></li>"
          )
        end

        it 'wraps multiple tags with container' do
          expect(
            subject.render_content(:text_list) do
              wrap :ul, :div, with: { class: 'wrapper foo' }
              wrap :em, :span, with: { class: 'special' }, if: -> { includes?('Bar') }
            end
          ).to eql(
            "<div class=\"wrapper foo\"><ul>\n"\
              "<li>Foo <span class=\"special\"><em>Bar</em></span></li>\n"\
              "<li>Baz</li>\n"\
            "</ul></div>\n"
          )
        end

        it 'wraps and modifies tags' do
          expect(
            subject.render_content(:text_list) do
              wrap :ul, :div, with: { class: 'container' }
              modify :em, with: { style: 'color: #c0ffee' }
            end
          ).to eql(
            "<div class=\"container\"><ul>\n"\
              "<li>Foo <em style=\"color: #c0ffee\">Bar</em></li>\n"\
              "<li>Baz</li>\n"\
            "</ul></div>\n"
          )
        end

        it 'replaces tags and adds class' do
          expect(
            subject.render_content(:test_content) do
              replace :p, :div, with: { class: 'byebye' }
              replace :em, :span, with: { id: 'i-am', class: 'cool', 'data-u' => 'r-not' }
            end
          ).to eql(
            "<h1>Content from <a href=\"http://somewhere.com\">somewhere</a></h1>\n\n"\
            "<div class=\"byebye\">"\
              "Lorem do <span id=\"i-am\" class=\"cool\" data-u=\"r-not\">incididunt</span> ut labore"\
            "</div>\n\n"\
            "<div class=\"byebye\">Pariatur: <a href=\"/bar.com\" title=\"baz\">Foo</a> culpa</div>\n"
          )
        end

        it 'wraps and replaces conditionally' do
          expect(
            subject.render_content(:test_content) do
              wrap :a, :span, with: { class: 'btn' }, if: { href: 'http://somewhere.com' }
              replace :p, :div, if: -> { includes?('Lorem') }
              replace :em, :span, with: { class: 'replaced' }, if: -> { includes?('incididunt') }
            end
          ).to eql(
            "<h1>Content from <span class=\"btn\"><a href=\"http://somewhere.com\">somewhere</a></span></h1>\n\n"\
            "<div>Lorem do <span class=\"replaced\">incididunt</span> ut labore</div>\n\n"\
            "<p>Pariatur: <a href=\"/bar.com\" title=\"baz\">Foo</a> culpa</p>\n"
          )
        end
      end

      describe 'with caching' do
        before(:each) { ClientPages.cache.clear }
        after { ClientPages.configure { |c| c.caching = false } }

        context 'enabled' do
          before do
            ClientPages.configure do |c|
              c.caching = true
              c.remote = true
              c.remote_content_path = 'http://c.ly'
            end
          end

          it 'populates cache when empty' do
            request = stub_request(:get, /\/caching.md/).to_return(body: '**cache me**')
            expect { subject.render_content(:caching) }
              .to change { ClientPages.cache }
              .to eql('http://c.ly/caching.md' => '**cache me**')
            expect(request).to have_been_made.times(1)
          end

          it 'makes request only once' do
            request = stub_request(:get, /\/caching.md/).to_return(body: '**cache me**')
            5.times { subject.render_content(:caching) }
            expect { subject.render_content(:caching) }.to_not change { ClientPages.cache }
            expect(request).to have_been_made.times(1)
          end
        end

        context 'disabled' do
          before do
            ClientPages.configure do |c|
              c.caching = false
              c.remote = true
              c.remote_content_path = 'http://c.ly'
            end
          end

          it 'will not cache' do
            stub_request(:get, /\/no_caching.md/).to_return(body: '_u_no_cache_me_')
            subject.render_content(:no_caching)
            expect(ClientPages.cache).to be_empty
          end

          it 'makes a request per render_content call' do
            request = stub_request(:get, /\/no_caching.md/).to_return(body: '_u_no_cache_me_')
            5.times { subject.render_content(:no_caching) }
            expect(request).to have_been_made.times(5)
          end
        end
      end

      describe 'default templates' do
        before do
          ClientPages.templating do
            template :basic do
              modify :h1, with: { class: 'headers' }
              wrap :a, :span, with: { class: 'links' }, if: -> { includes?('somewhere') }
              replace :p, :div
            end
          end
        end

        it 'uses templates and accepts additional block' do
          expect(
            subject.render_content(:test_content, template: :basic) do
              modify :h1, with: { 'data-foo' => 'bar' }
            end
          ).to eql(
            "<h1 class=\"headers\" data-foo=\"bar\">Content from <span class=\"links\">"\
            "<a href=\"http://somewhere.com\">somewhere</a></span></h1>\n\n"\
            "<div>Lorem do <em>incididunt</em> ut labore</div>\n\n<div>Pariatur: "\
            "<a href=\"/bar.com\" title=\"baz\">Foo</a> culpa</div>\n"
          )
        end
      end
    end

    describe 'configuration' do
      subject { ClientPages }
      before { subject.configuration = nil }

      it 'is configurable with a block' do
        expect {
          subject.configure do |c|
            c.content_path = 'somewhere'
            c.render_options = { foo: 'bar' }
            c.markdown_extensions = { bam: true }
          end
        }.to change {
          subject.config.to_hash
        }.to eql(
          content_path: 'somewhere',
          render_options: { foo: 'bar' },
          markdown_extensions: { bam: true }
        )
      end
    end
  end
end
